# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015, 2016, 2017 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.
"""Module for depositing record metadata and uploading files."""
import os
from typing import Any
from typing import Dict

from setuptools import find_packages
from setuptools import setup

readme = open("README.rst").read()
history = open("CHANGES.rst").read()

install_requires = [
    "bagit",
    "marshmallow",
    "rdflib",
    "rdflib-jsonld",
    "invenio-celery",
    "invenio-db[versioning]",
    "invenio-deposit",
    "invenio-files-rest",
    "invenio-jsonschemas",
    "invenio-records-files",
    "invenio-records-rest",
    "rfc6266-parser",
    "sword3common",
    # We use typing.Protocol, which is Py3.8+, but is available in typing-extensions for backwards compatibility
    'typing-extensions;python_version<"3.8"',
]

packages = find_packages()


# Get the version string. Cannot be done with import!
g: Dict[str, Any] = {}
with open(os.path.join("invenio_sword", "version.py"), "rt") as fp:
    exec(fp.read(), g)
    version = g["__version__"]

setup(
    name="invenio-sword",
    version=version,
    description=__doc__,
    long_description=readme + "\n\n" + history,
    long_description_content_type="text/x-rst",
    keywords="invenio sword deposit upload",
    license="GPLv2",
    author="CERN",
    author_email="info@inveniosoftware.org",
    url="https://github.com/inveniosoftware/invenio-sword",
    packages=packages,
    zip_safe=False,
    include_package_data=True,
    platforms="any",
    entry_points={
        "invenio_base.apps": ["invenio_sword = invenio_sword:InvenioSword",],
        "invenio_base.api_apps": ["invenio_sword = invenio_sword:InvenioSword",],
        "invenio_sword.packaging": [
            "http://purl.org/net/sword/3.0/package/Binary = invenio_sword.packaging:BinaryPackaging",
            "http://purl.org/net/sword/3.0/package/SimpleZip = invenio_sword.packaging:SimpleZipPackaging",
            "http://purl.org/net/sword/3.0/package/SWORDBagIt = invenio_sword.packaging:SWORDBagItPackaging",
        ],
        "invenio_sword.metadata": [
            "http://purl.org/net/sword/3.0/types/Metadata = invenio_sword.metadata:SWORDMetadata",
        ],
        "invenio_celery.tasks": ["invenio_sword = invenio_sword.tasks",],
    },
    install_requires=install_requires,
    extras_require={"test": ["pytest", "pytest-httpserver",]},
    classifiers=[
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: Implementation :: CPython",
        "Development Status :: 1 - Planning",
    ],
)



from http import HTTPStatus

from flask import request
from flask import Response
from invenio_db import db
from invenio_records_rest.views import need_record_permission
from invenio_records_rest.views import pass_record

from . import SWORDDepositView
from ..api import SWORDDeposit

__all__ = ["DepositFilesetView"]


class DepositFilesetView(SWORDDepositView):
    view_name = "{}_fileset"

    @pass_record
    @need_record_permission("update_permission_factory")
    def post(self, pid, record: SWORDDeposit):
        self.ingest_file(
            record,
            request.stream
            if (request.content_type or request.content_length)
            else None,
            replace=False,
        )
        record.commit()
        db.session.commit()
        return Response(status=HTTPStatus.NO_CONTENT)

    @pass_record
    @need_record_permission("update_permission_factory")
    def put(self, pid, record: SWORDDeposit):
        self.ingest_file(
            record,
            request.stream
            if (request.content_type or request.content_length)
            else None,
        )
        record.commit()
        db.session.commit()
        return Response(status=HTTPStatus.NO_CONTENT)

    @pass_record
    @need_record_permission("update_permission_factory")
    def delete(self, pid, record: SWORDDeposit):
        self.ingest_file(
            record, None,
        )
        record.commit()
        db.session.commit()
        return Response(status=HTTPStatus.NO_CONTENT)



import io
from http import HTTPStatus

from flask import url_for
from flask_security import url_for_security
from invenio_db import db
from invenio_files_rest.models import ObjectVersion
from invenio_files_rest.models import ObjectVersionTag

from invenio_sword.api import SWORDDeposit
from invenio_sword.enum import ObjectTagKey


def test_get_status_document_not_found(api, location, es):
    with api.app_context(), api.test_client() as client:
        response = client.get("/sword/deposit/1234")
        assert response.status_code == HTTPStatus.NOT_FOUND


def test_get_status_document(api, users, location, es):
    with api.test_request_context(), api.test_client() as client:
        client.post(
            url_for_security("login"),
            data={"email": users[0]["email"], "password": "tester"},
        )
        record = SWORDDeposit.create({})
        record.commit()
        db.session.commit()

        ObjectVersion.create(
            record.bucket,
            "file.n3",
            mimetype="text/n3",
            stream=io.BytesIO(b"1 _:a 2 ."),
        )

        response = client.get("/sword/deposit/{}".format(record.pid.pid_value))
        assert response.status_code == HTTPStatus.OK
        assert len(response.json["links"]) == 1
        assert response.json["links"][0]["contentType"] == "text/n3"


def test_put_status_document(api, users, location, es):
    with api.test_request_context(), api.test_client() as client:
        client.post(
            url_for_security("login"),
            data={"email": users[0]["email"], "password": "tester"},
        )
        record = SWORDDeposit.create({})
        record.commit()
        db.session.commit()

        object_version = ObjectVersion.create(
            record.bucket,
            "file.n3",
            mimetype="text/n3",
            stream=io.BytesIO(b"1 _:a 2 ."),
        )
        ObjectVersionTag.create(
            object_version=object_version,
            key=ObjectTagKey.FileSetFile.value,
            value="true",
        )

        response = client.put(
            "/sword/deposit/{}".format(record.pid.pid_value), data=b""
        )
        assert response.status_code == HTTPStatus.OK

        # This should have removed the previous file, as the empty PUT is a reset.
        object_versions = list(
            ObjectVersion.query.filter_by(bucket=record.bucket).order_by("created")
        )
        assert len(object_versions) == 2
        assert not object_versions[0].is_head
        assert object_versions[1].is_head
        assert object_versions[1].file is None


def test_cant_unpublish(api, users, location, es):
    with api.test_request_context(), api.test_client() as client:
        client.post(
            url_for_security("login"),
            data={"email": users[0]["email"], "password": "tester"},
        )

        response = client.post(url_for("invenio_sword.depid_service_document"))
        assert response.status_code == HTTPStatus.CREATED
        assert "http://purl.org/net/sword/3.0/state/ingested" in [
            state["@id"] for state in response.json["state"]
        ]

        response = client.post(
            response.headers["Location"], headers={"In-Progress": "true"}
        )
        assert response.status_code == HTTPStatus.CONFLICT


def test_delete_status_document(api, users, location, es):
    with api.test_request_context(), api.test_client() as client:
        client.post(
            url_for_security("login"),
            data={"email": users[0]["email"], "password": "tester"},
        )
        record = SWORDDeposit.create({})
        record.commit()
        db.session.commit()

        response = client.delete("/sword/deposit/{}".format(record.pid.pid_value))
        assert response.status_code == HTTPStatus.NO_CONTENT

        response = client.get("/sword/deposit/{}".format(record.pid.pid_value))
        assert response.status_code == HTTPStatus.GONE



from __future__ import annotations

from invenio_files_rest.models import ObjectVersion
from sword3common.constants import PackagingFormat

from ..enum import ObjectTagKey
from ..utils import TagManager
from .base import Packaging

__all__ = ["BinaryPackaging"]


class BinaryPackaging(Packaging):
    packaging_name = PackagingFormat.Binary

    def shortcut_unpack(self, object_version: ObjectVersion):
        tags = TagManager(object_version)
        tags[ObjectTagKey.FileSetFile] = "true"
        return []

    def unpack(self, object_version: ObjectVersion):
        return self.shortcut_unpack(object_version)



from .base import SWORDDepositView

from .file import DepositFileView
from .fileset import DepositFilesetView
from .metadata import DepositMetadataView
from .service_document import ServiceDocumentView
from .staging import StagingURLView, TemporaryURLView
from .status import DepositStatusView

from .blueprint import create_blueprint

__all__ = [
    "SWORDDepositView",
    "DepositFileView",
    "DepositFilesetView",
    "DepositMetadataView",
    "ServiceDocumentView",
    "DepositStatusView",
    "StagingURLView",
    "TemporaryURLView",
    "create_blueprint",
]



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016 CERN.
#
# Invenio is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Invenio is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Invenio; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# In applying this license, CERN does not
# waive the privileges and immunities granted to it by virtue of its status
# as an Intergovernmental Organization or submit itself to any jurisdiction.
"""Pytest helpers."""
from __future__ import absolute_import
from __future__ import print_function

import binascii
from copy import deepcopy

from flask_security import url_for_security
from six import BytesIO


def stringio_to_base64(stringio_obj):
    """Get base64 encoded version of a BytesIO object."""
    return binascii.b2a_base64(stringio_obj.getvalue())


def make_file_fixture(filename, base64_file):
    """Generate a file fixture suitable for use with the Flask test client.

    :param base64_file: A string encoding a file in base64. Use
        file_to_base64() to get the base64 encoding of a file. If not provided
        a PDF file be generated instead, including
    """
    fp = BytesIO(binascii.a2b_base64(base64_file))
    return fp, filename


def make_pdf_fixture(filename, text=None):
    """Generate a PDF fixture.

    It's suitable for use with Werkzeug test client and Flask test request
    context.
    Use of this function requires that reportlab have been installed.

    :param filename: Desired filename.
    :param text: Text to include in PDF. Defaults to "Filename: <filename>", if
        not specified.
    """
    if text is None:
        text = "Filename: %s" % filename

    # Generate simple PDF
    from reportlab.pdfgen import canvas

    output = BytesIO()
    c = canvas.Canvas(output)
    c.drawString(100, 100, text)
    c.showPage()
    c.save()

    return make_file_fixture(filename, stringio_to_base64(output))


def fill_oauth2_headers(json_headers, token):
    """Create authentication headers (with a valid oauth2 token)."""
    headers = deepcopy(json_headers)
    headers.append(("Authorization", "Bearer {0}".format(token.access_token)))
    return headers


def login(client, email="info@inveniosoftware.org", password="tester", **kwargs):
    client.post(
        url_for_security("login"), data={"email": email, "password": password},
    )



from http import HTTPStatus

from flask import Response
from invenio_db import db
from invenio_records_rest.views import need_record_permission
from invenio_records_rest.views import pass_record

from . import SWORDDepositView
from ..api import SWORDDeposit

__all__ = ["DepositStatusView"]


class DepositStatusView(SWORDDepositView):
    view_name = "{}_item"

    @pass_record
    @need_record_permission("read_permission_factory")
    def get(self, pid, record: SWORDDeposit):
        """Retrieve a SWORD status document for a deposit record

        :see also: https://swordapp.github.io/swordv3/swordv3.html#9.6.
        """
        return record.get_status_as_jsonld()

    @pass_record
    @need_record_permission("update_permission_factory")
    def post(self, pid, record: SWORDDeposit):
        """Augment a SWORD deposit with either metadata or files"""
        self.update_deposit(record, replace=False)
        return record.get_status_as_jsonld()

    @pass_record
    @need_record_permission("update_permission_factory")
    def put(self, pid, record: SWORDDeposit):
        """Replace a SWORD deposit with either metadata or files"""
        self.update_deposit(record)
        return record.get_status_as_jsonld()

    @pass_record
    @need_record_permission("update_permission_factory")
    def delete(self, pid, record: SWORDDeposit):
        """Delete a SWORD deposit"""
        record.delete()
        db.session.commit()
        return Response(status=HTTPStatus.NO_CONTENT)



import enum


class ObjectTagKey(enum.Enum):
    OriginalDeposit = "invenio_sword.originalDeposit"
    DerivedFrom = "invenio_sword.derivedFrom"
    FileSetFile = "invenio_sword.fileSetFile"
    Packaging = "invenio_sword.packaging"
    MetadataFormat = "invenio_sword.metadataFormat"
    FileState = "invenio_sword.fileState"
    ByReferenceURL = "invenio_sword.byReferenceURL"
    ByReferenceTemporaryID = "invenio_sword.byReferenceTemporaryID"
    ByReferenceDereference = "invenio_sword.byReferenceDereference"
    ByReferenceTTL = "invenio_sword.byReferenceTTL"
    ByReferenceContentLength = "invenio_sword.byReferenceContentLength"
    # Used to mark an object version as extent, even though it's not got a file.
    ByReferenceNotDeleted = "invenio_sword.byReferenceNotDeleted"


class FileState(enum.Enum):
    Pending = "http://purl.org/net/sword/3.0/filestate/pending"
    Downloading = "http://purl.org/net/sword/3.0/filestate/downloading"
    Unpacking = "http://purl.org/net/sword/3.0/filestate/unpacking"
    Error = "http://purl.org/net/sword/3.0/filestate/error"
    Ingested = "http://purl.org/net/sword/3.0/filestate/ingested"
